import ssl
import json
import yaml
import time
from awscrt import io, mqtt, auth, http
from awsiot import mqtt_connection_builder


from datetime import datetime, timezone, timedelta

class sdtcloudpubsub():
    def __init__(self):
        appId, deviceConfig = self.getConfig()
        self.url = "avk03ee629rck-ats.iot.ap-northeast-2.amazonaws.com"
        self.certificate = f"/etc/sdt/cert/{deviceConfig['projectcode']}-certificate.pem"
        self.privateKey = f"/etc/sdt/cert/{deviceConfig['projectcode']}-private.pem"
        self.rootCa = f"/etc/sdt/cert/AmazonRootCA1.pem"
        self.topic = f"{deviceConfig['servicecode']}/{deviceConfig['projectcode']}/{deviceConfig['assetcode']}/app/{appId}/data"

    def getConfig(self):

        with open('/etc/sdt/device.config/app.json', encoding='UTF-8') as f:
            appConfig = json.load(f)
        
        with open('./framework.yaml', encoding='UTF-8') as f:
            appFramework = yaml.load(f, Loader=yaml.FullLoader)
        
        appId = ""
        for appInfo in appConfig["AppInfoList"]:
            if appInfo["AppName"] == appFramework["spec"]["appName"]:
                appId = appInfo["AppId"]

        with open('/etc/sdt/device.config/config.json', encoding='UTF-8') as f:
            deviceConfig = json.load(f)

        return appId, deviceConfig
    

    def setClient(self, clientId):
        eventLoopGroup = io.EventLoopGroup(1)
        hostResolver = io.DefaultHostResolver(eventLoopGroup)
        clientBootstrap = io.ClientBootstrap(eventLoopGroup, hostResolver)
        mqttClient = mqtt_connection_builder.mtls_from_path(
            endpoint=self.url,
            cert_filepath=self.certificate,
            pri_key_filepath=self.privateKey,
            client_bootstrap=clientBootstrap,
            ca_filepath=self.rootCa,
            client_id=clientId,
            clean_session=True,
            keep_alive_secs=60)

        return mqttClient

    def pubMessage(self, mqttClient, msg):
        # Make the connect() call
        connectFuture = mqttClient.connect()
        
        # Future.result() waits until a result is available
        connectFuture.result()
        
        # Publish message to server desired number of times.
        payload = {
            "data": msg,
            "timestamp": int(time.time()*1000)
        }
        mqttClient.publish(topic=self.topic, payload=json.dumps(payload), qos=mqtt.QoS.AT_LEAST_ONCE)
        

        disconnectFuture = mqttClient.disconnect()
        disconnectFuture.result()

